export declare const InternalTypeSymbol: unique symbol;
export declare type JsonValue = {
    [property: string]: JsonValue;
} | boolean | JsonValue[] | null | number | string;
declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends (k: infer I) => void ? I : never;
interface BaseJsonSchema {
    title?: string;
    description?: string;
}
interface StringOptions<EnumType extends string | undefined = undefined> extends BaseJsonSchema {
    enum?: readonly EnumType[];
    format?: string;
    maxLength?: string;
    minLength?: string;
    pattern?: string;
}
interface NumberOptions extends BaseJsonSchema {
    multipleOf?: number;
    minimum?: number;
    exclusiveMinimum?: number;
    maximum?: number;
    exclusiveMaximum?: number;
    description?: string;
}
interface ObjectOptions<Properties extends {
    [k: string]: JsonSchema;
}, AdditionalProperties extends JsonSchema, // TODO support boolean
RequiredProperties extends readonly (keyof Properties)[]> extends BaseJsonSchema {
    properties?: Properties;
    additionalProperties?: AdditionalProperties;
    required: RequiredProperties;
    propertyNames?: JsonSchema;
    minProperties?: number;
    maxProperties?: number;
    patternProperties?: {
        [k: string]: JsonSchema;
    };
}
interface ArrayOptions<Items, // extends JsonSchema | readonly JsonSchema[],
AdditionalItems extends Items extends JsonSchema ? never : JsonSchema> extends BaseJsonSchema {
    items?: Items;
    additionalItems?: AdditionalItems;
    minItems?: number;
    maxItems?: number;
    uniqueItems?: boolean;
}
interface BooleanOptions extends BaseJsonSchema {
}
interface NullOptions extends BaseJsonSchema {
}
interface AnyOfOptions<Schemas extends readonly JsonSchema[]> extends BaseJsonSchema {
    anyOf: Schemas;
}
interface AllOfOptions<Schemas extends readonly JsonSchema[]> extends BaseJsonSchema {
    allOf: Schemas;
}
export interface InterfaceWithHiddenType {
    [InternalTypeSymbol]: unknown;
}
export interface JsonSchema extends InterfaceWithHiddenType {
}
export declare const Schema: {
    String: <T extends string>(options?: StringOptions<T>) => {
        [InternalTypeSymbol]: T;
        enum?: readonly T[] | undefined;
        format?: string | undefined;
        maxLength?: string | undefined;
        minLength?: string | undefined;
        pattern?: string | undefined;
        title?: string | undefined;
        description?: string | undefined;
        type: "string";
    };
    Number: (options?: NumberOptions) => {
        [InternalTypeSymbol]: number;
        multipleOf?: number | undefined;
        minimum?: number | undefined;
        exclusiveMinimum?: number | undefined;
        maximum?: number | undefined;
        exclusiveMaximum?: number | undefined;
        description?: string | undefined;
        title?: string | undefined;
        type: "number";
    };
    Integer: (options?: NumberOptions) => {
        [InternalTypeSymbol]: number;
        multipleOf?: number | undefined;
        minimum?: number | undefined;
        exclusiveMinimum?: number | undefined;
        maximum?: number | undefined;
        exclusiveMaximum?: number | undefined;
        description?: string | undefined;
        title?: string | undefined;
        type: "integer";
    };
    Object: <Properties extends {
        [k: string]: JsonSchema;
    }, AdditionalProperties extends JsonSchema, RequiredProperties extends readonly (keyof Properties)[]>(options: ObjectOptions<Properties, AdditionalProperties, RequiredProperties>) => {
        [InternalTypeSymbol]: Partial<{ [P in keyof Properties]: Properties[P][typeof InternalTypeSymbol]; }> & Required<Pick<Partial<{ [P in keyof Properties]: Properties[P][typeof InternalTypeSymbol]; }>, RequiredProperties[number]>> & {
            [k: string]: AdditionalProperties;
        };
        properties?: Properties | undefined;
        additionalProperties?: AdditionalProperties | undefined;
        required: RequiredProperties;
        propertyNames?: JsonSchema | undefined;
        minProperties?: number | undefined;
        maxProperties?: number | undefined;
        patternProperties?: {
            [k: string]: JsonSchema;
        } | undefined;
        title?: string | undefined;
        description?: string | undefined;
        type: "object";
    };
    Array: <Items extends JsonSchema | readonly JsonSchema[], AdditionalItems extends Items extends JsonSchema ? never : JsonSchema>(options?: ArrayOptions<Items, AdditionalItems>) => {
        [InternalTypeSymbol]: Items extends readonly JsonSchema[] ? (Items[number][typeof InternalTypeSymbol] | AdditionalItems[typeof InternalTypeSymbol])[] : Items extends JsonSchema ? Items[typeof InternalTypeSymbol][] : never;
        items?: Items | undefined;
        additionalItems?: AdditionalItems | undefined;
        minItems?: number | undefined;
        maxItems?: number | undefined;
        uniqueItems?: boolean | undefined;
        title?: string | undefined;
        description?: string | undefined;
        type: "array";
    };
    Boolean: (options?: BooleanOptions) => {
        [InternalTypeSymbol]: boolean;
        title?: string | undefined;
        description?: string | undefined;
        type: "boolean";
    };
    Null: (options?: NullOptions) => {
        [InternalTypeSymbol]: null;
        title?: string | undefined;
        description?: string | undefined;
        type: "null";
    };
    AnyOf: <Schemas extends readonly JsonSchema[]>(options: AnyOfOptions<Schemas>) => {
        [InternalTypeSymbol]: Schemas[number][typeof InternalTypeSymbol];
        anyOf: Schemas;
        title?: string | undefined;
        description?: string | undefined;
    };
    AllOf: <Schemas_1 extends readonly JsonSchema[]>(options: AllOfOptions<Schemas_1>) => {
        [InternalTypeSymbol]: UnionToIntersection<Schemas_1[number][typeof InternalTypeSymbol]>;
        allOf: Schemas_1;
        title?: string | undefined;
        description?: string | undefined;
    };
};
export {};
